{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }
{                                                                              }                                                                              
{ Filename           : DFWDriveCombo.pas                                       }
{ Short description  :                                                         }
{ Author             : Jonathan Faivre-Vuillin                                 }
{ E-Mail             : Lapince@Ifrance.com                                     }
{ Internet           :                                                         }
{ Version            : 1.1                                                     }
{ Date last modified : 15/04/2000                                              }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     : TDFWDriveCombo based on TDriveComboBox from             }
{                      Peter Thrnqvist                                        }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWDriveCombo                                            }
{ Classes          :                                                           }
{ Types            : TWMDeviceChange, PDEV_BROADCAST_HDR, TDEV_BROADCAST_HDR,  }
{                    PDEV_BROADCAST_VOLUME, TDEV_BROADCAST_VOLUME, TImageSize, }
{                    TMediaEvent, TIncludedMedias, TExcludedDrives             }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWDriveCombo;

{$IFNDEF VER80}
 {$IFNDEF VER90}
  {$IFNDEF VER93}
    {$IFNDEF VER100}
      {$IFNDEF VER110}
        {$DEFINE ST4}
        {$IFNDEF VER120}
           {$IFNDEF VER125}
            {$DEFINE ST5}
          {$ENDIF}
        {$ENDIF}
      {$ENDIF}
    {$ENDIF}
  {$ENDIF}
 {$ENDIF}
{$ENDIF}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, {$IFDEF ST4} ImgList, {$ENDIF} ShellAPI, extctrls;

const
      ComponentVersion='1.1';
      ComponentAuthor='Jonathan Faivre-Vuillin';
      ComponentEMail='Lapince@Ifrance.com';
      ComponentHomePage='';

type
  TWMDeviceChange = record
     Msg : Cardinal;
     Event : UINT;
     dwData : Pointer;
     Result : LongInt;
    end;

  //When we got a DBT_DEVICECHANGE or DBT_DEVICEREMOVECOMPLETE event, dwData contains an address of a DEV_BROADCAST_HDR structure identifying the device inserted. We also need a pointer so we can point to this structure.

  PDEV_BROADCAST_HDR = ^TDEV_BROADCAST_HDR;
   TDEV_BROADCAST_HDR = packed record
     dbch_size : DWORD;
     dbch_devicetype : DWORD;
     dbch_reserved : DWORD;
    end;

  //When the device is of type volume, then we can get some device specific information, namely specific information about a logical volume.

  PDEV_BROADCAST_VOLUME = ^TDEV_BROADCAST_VOLUME;
    TDEV_BROADCAST_VOLUME = packed record
     dbcv_size : DWORD;
     dbcv_devicetype : DWORD;
     dbcv_reserved : DWORD;
     dbcv_unitmask : DWORD;
     dbcv_flags : WORD;
    end;

const

(* Events of WM_DEVICECHANGE (wParam) *)

  DBT_DEVICEARRIVAL = $8000;            (* system detected a new device *)
  DBT_DEVICEQUERYREMOVE = $8001;        (* wants to remove, may fail *)
  DBT_DEVICEQUERYREMOVEFAILED = $8002;  (* removal aborted *)
  DBT_DEVICEREMOVEPENDING = $8003;      (* about to remove, still avail *)
  DBT_DEVICEREMOVECOMPLETE = $8004;     (* device is gone *)
  DBT_DEVICETYPESPECIFIC = $8005;       (* type specific event *)
  DBT_CONFIGCHANGED = $0018;

 (* type of device in DEV_BROADCAST_HDR *)
  DBT_DEVTYP_OEM = $00000000;           (* OEM- or IHV-defined *)
  DBT_DEVTYP_DEVNODE = $00000001;       (* Devnode number *)
  DBT_DEVTYP_VOLUME = $00000002;        (* Logical volume *)
  DBT_DEVTYP_PORT = $00000003;          (* Port (serial or parallel *)
  DBT_DEVTYP_NET = $00000004;           (* Network resource *)

  (* media types in DBT_DEVTYP_VOLUME *)
  DBTF_MEDIA = $0001;                (* change affects media in drive *)
  DBTF_NET = $0002;                     (* logical volume is network volume *)



type
  TImageSize=(isSmall,isLarge);
  TMediaEvent=procedure(Media: char) of object;
  TIncludedMedias=set of (imRemovable, imFixed, imCDROM, imRemote, imRamdisk);
  //TExcludedDrives=set of 'a'..'z';
  TExcludedDrives=set of 'a'..'z';

//--- I may use that in a future improvement
{  TMediaPresentDisplayInfo=(mpCustomString,mpName,mpType);
  //--- When a media is present in a removable drive... display :
  //  mpCustomString -> The string in "FRemovableString"
  //  mpName         -> The media volume name (if it has)
  //  mpType         -> The media type (if it has)

  TNoMediaDisplayInfo=(maSysName,maCustomString);
  //-- When no media in a removable drive... display :
  //  maSysName      -> The name returned by SHGetFileInfo
  //  maCustomString -> The string in "FRemovableString"
 }
  TDFWDriveCombo = class(TCustomComboBox)
  private
    { Private declarations }
    FDrives:TStrings;
    FImages:TImagelist;
    FImageWidth:integer;
    FImageSize:TImageSize;
    FItemIndex:integer;
    FOffset:integer;
    FDrive:char;
    FSmall,FLarge:integer;
    FTimer: TTimer;
    FOwnWndProcHandle: THandle;
    fOnMediaRemove, fOnMediaArrival: TMediaEvent;

    FRemovableName: string;
    FIncludedMedias: TIncludedMedias;
    FExcludedDrives: TExcludedDrives;

//--- I may use that in a future improvement
//    FUpdateRemovable: Boolean;
//    FRemovablePresentDisplay: TMediaPresentDisplayInfo;
    // If mpName or mpType selected and SHGetFileInfo returns no type or name, the Custom string is used

//    FRemovableAbsentDisplay: TNoMediaDisplayInfo;
    procedure TimerProc(Sender: TObject);

    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure ResetItemHeight;
    procedure SetImageSize(Value:TImageSize);
    procedure SetOffset(Value:integer);
    procedure OwnWndProc(var Message: TMessage);
  protected
    { Protected declarations }
    FOnChange: TNotifyEvent;
    Initializing: Boolean;
    FSaveDrive: char;
    procedure CreateWnd; override;
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
    procedure SetDrive(Value:char);
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure MeasureItem(Index: Integer;var Height: Integer);override;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
    procedure Change; override;

    procedure SetExcludedDrives(Value: TExcludedDrives);
    procedure SetIncludedMedias(Value: TIncludedMedias);
  public
    { Public declarations }
    procedure BuildList; //virtual;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
  published
    { Published declarations }
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    property Drive: char read FDrive write SetDrive;
    property Offset:integer read FOffset write SetOffset;
    property ImageSize: TImageSize read FImageSize write SetImageSize;
    property RemovableName: string read FRemovableName write FRemovableName;

    {$IFDEF ST4}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
    property DropDownCount;
    {$ENDIF}
    {$IFDEF ST5}
    property OnConTextPopup;
    {$ENDIF}
    {$IFNDEF VER90}
    property ImeMode;
    property ImeName;
    {$ENDIF}

    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;

    property OnMediaRemove: TMediaEvent read fOnMediaRemove write fOnMediaRemove;
    property OnMediaArrival: TMediaEvent read fOnMediaArrival write fOnMediaArrival;

    property ExcludedDrives: TExcludedDrives read FExcludedDrives write SetExcludedDrives;
    property IncludedMedias: TIncludedMedias read FIncludedMedias write SetIncludedMedias;
  end;

implementation

function GetItemHeight(Font: TFont): Integer;
var DC: HDC; SaveFont: HFont; Metrics: TTextMetric;
begin
  DC := GetDC(0);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics(DC, Metrics);
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
  Result := Metrics.tmHeight+2;
end;

function GetFirstDriveLetter(unitmask : longint):char;
var
  DriveLetter : shortint;
begin
  DriveLetter := Ord('A');
  while (unitmask and 1)=0  do begin
    unitmask := unitmask shr 1;
    inc(DriveLetter);
  end;
  Result := Char(DriveLetter);
end;

{ TDFWDriveCombo }

procedure TDFWDriveCombo.Loaded;
begin
  inherited Loaded;
//  SetDrive (FDrive);
end;

procedure TDFWDriveCombo.SetExcludedDrives(Value: TExcludedDrives);
begin
  if Value<>FExcludedDrives then
  begin
    FExcludedDrives:=Value;
    BuildList;
  end;
end;

procedure TDFWDriveCombo.SetIncludedMedias(Value: TIncludedMedias);
begin
  if Value<>FIncludedMedias then
  begin
    FIncludedMedias:=Value;
    BuildList;
  end;
end;

procedure TDFWDriveCombo.TimerProc(Sender: TObject);
begin
  //--- Insert a delay between the WM_DEVICECHANGE message and the update else the icons are bad drawn.
  // (Explorer does the same...)
  FTimer.Enabled:=False;
  BuildList;
end;

//--- When a media is to be removed, has been removed or inserted, this message is sent (See UNDU March 1998)
procedure TDFWDriveCombo.OwnWndProc(var Message: TMessage);
var
  lpdb : PDEV_BROADCAST_HDR;
  lpdbv : PDEV_BROADCAST_VOLUME;
  Msg: TWMDeviceChange;
begin
  Msg:=TWMDeviceChange(Message);
  (* received a wm_devicechange message *)
  lpdb := PDEV_BROADCAST_HDR(Msg.dwData);
  (* look at the event send together with the wm_devicechange message *)
  case Msg.Event of
    DBT_DEVICEARRIVAL :
      if lpdb^.dbch_devicetype = DBT_DEVTYP_VOLUME then
      begin
        lpdbv := PDEV_BROADCAST_VOLUME(Msg.dwData);
        if (lpdbv^.dbcv_flags and DBTF_MEDIA) = 1 then
        begin
          FTimer.Enabled:=True;
          if Assigned(fOnMediaArrival) then
            fOnMediaArrival(GetFirstDriveLetter(lpdbv^.dbcv_unitmask));
        end;
      end;
    DBT_DEVICEREMOVECOMPLETE :
      if lpdb^.dbch_devicetype = DBT_DEVTYP_VOLUME then
      begin
        lpdbv := PDEV_BROADCAST_VOLUME(Msg.dwData);
        if (lpdbv^.dbcv_flags and DBTF_MEDIA) = 1 then
        begin
          FTimer.Enabled:=True;
          if Assigned(fOnMediaRemove) then
            fOnMediaRemove(GetFirstDriveLetter(lpdbv^.dbcv_unitmask));
        end;
      end;
  end;
  inherited;
end;
//--- End of add

constructor TDFWDriveCombo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOwnWndProcHandle:=AllocateHwnd(OwnWndProc);
  FTimer:=TTimer.Create(Self);
  FTimer.Enabled:=False;
  FTimer.Interval:=2000;
  FTimer.OnTimer:=TimerProc;

  FLarge := GetSystemMetrics(SM_CXICON);
  FSmall := GetSystemMetrics(SM_CXSMICON);

  FDrives := TStringList.Create;

  if FImageSize = isSmall then
    FImages := TImagelist.CreateSize(FSmall,FSmall)
  else
    FImages := TImagelist.CreateSize(FLarge,FLarge);

  FImages.DrawingStyle := dsTransparent;
  FImageWidth := FImages.Width;
  FImages.ShareImages := True;

  FItemIndex := 0;
  FOffset := 4;
  Style := csOwnerDrawFixed;
  ResetItemHeight;

  FIncludedMedias:=[imRemovable, imCDROM, imRamdisk, imRemote, imFixed];
  FExcludedDrives:=['a','b'];
end;

destructor TDFWDriveCombo.Destroy;
begin
  FDrives.Free;
  FImages.Free;
  FTimer.Free;
  DeallocateHwnd(FOwnWndProcHandle);
  inherited Destroy;
end;

procedure TDFWDriveCombo.BuildList;
var
  Info: TSHFileInfo;
  Ch: char;
  S: string;
  i, Options: integer;

  RootPathName: PChar;
  VolumeName: array[0..255] of Char;
  VolumeSerialNumber: DWord;
  MaximumComponentLength: DWord;
  FileSystemFlags: DWord;
  FileSystemName: array[0..10] of Char;
  Len: Integer;
  OutStr: string;

  DriveName: string;
begin
  if Items.Count > 0 then
  begin
   if  ItemIndex <> -1 then
     FItemIndex := ItemIndex;
   Items.Clear;
   FDrives.Clear;
  end;

  Options := SHGFI_SYSICONINDEX;
  if FImageSize = isSmall then
  	Options := Options or SHGFI_SMALLICON
  else
   Options := Options or SHGFI_LARGEICON;

  FImages.Handle := SHGetFileInfo('',0,Info,SizeOf(TShFileInfo),Options);

  for ch := 'A' to 'Z' do
  begin
    S := ch + ':\';
    i := GetDriveType(PChar(S));
    case i of
      DRIVE_REMOVABLE: if not (imRemovable in FIncludedMedias) then continue;
      DRIVE_FIXED: if not (imFixed in FIncludedMedias) then continue;
      DRIVE_REMOTE: if not (imRemote in FIncludedMedias) then continue;
      DRIVE_CDROM: if not (imCDROM in FIncludedMedias) then continue;
      DRIVE_RAMDISK: if not (imRAMDISK in FIncludedMedias) then continue;
    end;


    if i in [0,1] then Continue; // Invalid
      Info.szDisplayName:='';
    SHGetFileInfo(PChar(S),0,Info,SizeOf(TShFileInfo),SHGFI_DISPLAYNAME or Options);

    if (Ch='A') or (Ch='B') then
      DriveName:=Info.szDisplayName
    else
      if I<>DRIVE_REMOVABLE then
      begin
        RootPathName:=Pchar(Ch+':\');
        VolumeName:='';
        GetVolumeInformation(RootPathName, VolumeName, SizeOf(VolumeName), @VolumeSerialNumber, MaximumComponentLength, FileSystemFlags, FileSystemName, SizeOf(FileSystemName));// then
        if VolumeName<>'' then
        begin
          DriveName:='';
          for Len:=0 to 255 do
            if VolumeName[Len]<>#0 then
              DriveName:=DriveName+VolumeName[Len]
            else
              break;
            DriveName:=DriveName[1]+LowerCase(Copy(DriveName,2,Length(DriveName)-1))+' ('+Ch+':)';
        end
        else
          DriveName:=' ('+UpperCase(Ch)+':)';
      end
      else // I=DRIVE_REMOVABLE
        if Info.szDisplayName='' then
          DriveName:=FRemovableName+' ('+UpperCase(Ch)+':)'
        else
          DriveName:=Info.szDisplayName;
{        if not FUpdateRemovable then
          if FRemovableAbsentDisplay=maSysName then
            DriveName:=Info.szDisplayName
          else
            DriveName:=FRemovableString+' ('+UpperCase(Ch)+':)'
        else
        begin
{          if (FRemovableAbsentDisplay=mpName) then
          begin
            RootPathName:=Pchar(Ch+':\');
            VolumeName:='';
            GetVolumeInformation(RootPathName, VolumeName, SizeOf(VolumeName), @VolumeSerialNumber, MaximumComponentLength, FileSystemFlags, FileSystemName, SizeOf(FileSystemName));// then
            if VolumeName<>'' then
            begin
              DriveName:='';
              for Len:=0 to 255 do
                if VolumeName[Len]<>#0 then
                  DriveName:=DriveName+VolumeName[Len]
                else
                  break;
                DriveName:=DriveName[1]+LowerCase(Copy(DriveName,2,Length(DriveName)-1))+' ('+Ch+':)';
              end
              else
                if FRemovableAbsentDisplay=maSysName then
                  DriveName:=Info.szDisplayName
                else
                  DriveName:=FRemovableString+' ('+UpperCase(Ch)+':)';
            end;
          end;}
    Items.AddObject(DriveName,TObject(Info.iIcon));
    FDrives.Add(ch);
  end;
  if not Initializing then
  begin
    if FItemIndex<FDrives.Count then
      SetDrive(FDrives[FItemIndex][1])
    else
      SetDrive(FDrives[FDrives.Count-1][1]);
  end;
  Update;
end;

procedure TDFWDriveCombo.CreateWnd;
begin
  inherited CreateWnd;
  Initializing:=true;
  BuildList;
  SetDrive(FDrive);
  Initializing:=false;
  if (csDesigning in ComponentState) then
  	SetDrive(GetCurrentDir[1]);
end;

procedure TDFWDriveCombo.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  Offset,i: Integer;
begin
  with Canvas do
  begin
    FillRect(Rect);
    Offset := FImageWidth + FOffset + FOffset;
    if FImages.Count > 0 then
    begin
      i := integer(Items.Objects[Index]);
      FImages.Draw(Canvas,Rect.Left + FOffset,Rect.Top+1,i);
      Rect.Left := Rect.Left + Offset;
    end;
    DrawText(Canvas.Handle, PChar(Items[Index]), -1, Rect,
             DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
  end;
end;

procedure TDFWDriveCombo.MeasureItem(Index: Integer;var Height: Integer);
begin
  Height:= ItemHeight;
end;

procedure TDFWDriveCombo.CMFontChanged(var Message: TMessage);
begin
  inherited;
  ResetItemHeight;
  RecreateWnd;
end;

procedure TDFWDriveCombo.ResetItemHeight;
var
  nuHeight: Integer;
begin
  nuHeight :=  GetItemHeight(Font);
  if nuHeight < (FImageWidth + 2) then nuHeight := FImageWidth + 2;
  ItemHeight := nuHeight;
end;

procedure TDFWDriveCombo.SetDrive(Value:char);
var
  i,j: integer;
begin
  if Items.Count<1 then Exit;
  j := 0;
  if FItemIndex <> -1 then
    j := FItemIndex;

  Value := UpCase(Value);
  If (FDrive <> Value) or ((FDrive=Value) and (j>Items.Count)) then
  begin
    for i := FDrives.Count - 1 downto 0 do
      if FDrives[i][1] = Value then
      begin
      	FDrive := Value;
        FItemIndex := i;
        ItemIndex := i;
        Exit;
      end;
  end
  else
  begin
    ItemIndex := j;
    Exit;
  end;
  ItemIndex:=Items.Count-1;
end;

procedure TDFWDriveCombo.SetImageSize(Value:TImageSize);
var
  i: integer;
begin
  if FImageSize <> Value then
  begin
    FIMageSize := Value;
    i := 0;
    if FItemIndex <> -1 then
      i := FItemIndex;

    if Items.Count > 0 then Items.Clear;
    if Assigned(FImages) then FImages.Free;

    if Value = isSmall then
      FImages := TImagelist.CreateSize(FSmall,FSmall)
    else
      FImages := TImagelist.CreateSize(FLarge,FLarge);

    FImages.DrawingStyle := dsTransparent;
    FImages.ShareImages := True;
    FImageWidth := FImages.Width;
    ResetItemHeight;
    RecreateWnd;
    BuildList;
    Change;
  end;
end;

procedure TDFWDriveCombo.SetOffset(Value:integer);
begin
  if FOffset <> Value then
  begin
    FOffset := Value;
    Refresh;
  end;
end;

procedure TDFWDriveCombo.Change;
begin
  if ItemIndex <> -1 then
    FItemIndex := ItemIndex;
  if FItemIndex<FDrives.Count then
    SetDrive(FDrives[FItemIndex][1])
  else
    SetDrive(FDrives[FDrives.Count-1][1]);

 if Assigned(FOnChange) then FOnChange(Self);
end;


procedure TDFWDriveCombo.CNCommand(var Message: TWMCommand);
begin
  inherited;
  case Message.NotifyCode of
    CBN_SELCHANGE:
        Change;
  end;
end;

function TDFWDriveCombo.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWDriveCombo.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWDriveCombo.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWDriveCombo.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.
