{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }
{                                                                              }
{ Filename           : DFWGrafxes.pas                                          }
{ Short description  :                                                         }
{ Author             : Erol S. Uzuner                                          }
{ E-Mail             : sarcon@tzi.de                                           }
{ Internet           :                                                         }
{ Version            : 1.0                                                     }
{ Date last modified : 05.26.96                                                }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     : Copyright (C) 1996-2000                                 }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       :                                                           }
{ Classes          :                                                           }
{ Types            : TFillDirection, TMyColor                                  }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures: DrawTiled, BitmapRotateHorizontal, BitmapRotateVertical,  }
{                    BitmapRotate90, DrawFrameBorder, DrawTransparentBitmap,   }
{                    BitmapGray, GradientFillRect                              }
{ Global functions : Rechteck, DarkerColor, LighterColor                       }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWGrafxes;

interface

uses  WinTypes, Graphics, ExtCtrls, Classes;

procedure DrawTiled(DC : HDC; CR: TRect; Bitmap : TBitmap);
procedure BitmapRotateHorizontal(Bitmap : TBitmap);
procedure BitmapRotateVertical(Bitmap : TBitmap);
procedure BitmapRotate90(Bitmap : TBitmap);
procedure DrawFrameBorder(Canvas: TCanvas; X, Y, W, H : Integer);
procedure DrawTransparentBitmap( canvas: TCanvas; bmp: TBitmap;
                                 fromRect, toRect : TRect;
                                 cTransparentColor: LongInt);
function Rechteck(left, top, right, bottom : integer) : TRect;
procedure BitmapGray(Bitmap : TBitmap);
function DarkerColor(c1 : TColor; darkness: Byte): TColor;
function LighterColor(c1 : TColor; lightness: Byte): TColor;

{ Gradient filling routine }

type
  TFillDirection = (fdTopToBottom, fdBottomToTop, fdLeftToRight, fdRightToLeft);

  TMyColor = record
     case Boolean of
        TRUE :  (C : TColor);
        FALSE :  ( R, G, B, A : Byte);
       // FALSE :  ( G, R, B, A : Byte);
  end;

procedure GradientFillRect(Canvas: TCanvas; Rect: TRect; BeginColor,
  EndColor: TColor; Direction: TFillDirection; Colors: Byte);

implementation
{$IFNDEF D2}uses Math;{$ENDIF}
const
{ TBitmap.GetTransparentColor from GRAPHICS.PAS use this value }
  PaletteMask = $02000000;

function WidthOf(R: TRect): Integer;
begin Result := R.Right - R.Left;
end;

function HeightOf(R: TRect): Integer;
begin Result := R.Bottom - R.Top;
end;

function LighterColor(c1 : TColor; lightness: Byte): TColor;
var C:TmyColor;
begin
   C.R := (TmyColor(c1).R + lightness) MOD 255;
   C.B := (TmyColor(c1).B + lightness) MOD 255;
   C.G := (TmyColor(c1).G + lightness) MOD 255;
   C.A := TmyColor(c1).A;//(TmyColor(c1).A - darkness) MOD 255;
   Result := C.C;
end;

function DarkerColor(c1 : TColor; darkness: Byte): TColor;
var C:TmyColor;
begin
   C.R := (TmyColor(c1).R - darkness) MOD 255;
   C.B := (TmyColor(c1).B - darkness) MOD 255;
   C.G := (TmyColor(c1).G - darkness) MOD 255;
   C.A := TmyColor(c1).A;//(TmyColor(c1).A - darkness) MOD 255;
   Result := C.C;
end;

{ Gradient fill procedure - displays a gradient beginning with a chosen    }
{ color and ending with another chosen color. Based on TGradientFill       }
{ component source code written by Curtis White, cwhite@teleport.com.      }
procedure GradientFillRect(Canvas: TCanvas; Rect: TRect; BeginColor,
  EndColor: TColor; Direction: TFillDirection; Colors: Byte);
{$IFDEF D2}
function Max(A,B: Integer): Integer;
begin
  if A > B then
    Result := A
  else
    Result := B;
end;
function Min(A,B: Integer): Integer;
begin
  if A < B then
    Result := A
  else
    Result := B;
end;

{$ENDIF}
var
  BeginRGBValue: array[0..2] of Byte;    { Begin RGB values }
  RGBDifference: array[0..2] of Integer; { Difference between begin and end RGB values }
  ColorBand: TRect; { Color band rectangular coordinates }
  I, Delta: Integer;
  R, G, B: Byte;    { Color band Red, Green, Blue values }
begin
  if IsRectEmpty(Rect) then Exit;
  BeginColor := ColorToRGB(BeginColor);
  EndColor := ColorToRGB(EndColor);
  case Direction of
    fdTopToBottom, fdLeftToRight: begin
      { Set the Red, Green and Blue colors }
      BeginRGBValue[0] := GetRValue(BeginColor);
      BeginRGBValue[1] := GetGValue(BeginColor);
      BeginRGBValue[2] := GetBValue(BeginColor);
      { Calculate the difference between begin and end RGB values }
      RGBDifference[0] := GetRValue(EndColor) - BeginRGBValue[0];
      RGBDifference[1] := GetGValue(EndColor) - BeginRGBValue[1];
      RGBDifference[2] := GetBValue(EndColor) - BeginRGBValue[2];
    end;
    fdBottomToTop, fdRightToLeft: begin
      { Set the Red, Green and Blue colors }
      { Reverse of TopToBottom and LeftToRight directions }
      BeginRGBValue[0] := GetRValue(EndColor);
      BeginRGBValue[1] := GetGValue(EndColor);
      BeginRGBValue[2] := GetBValue(EndColor);
      { Calculate the difference between begin and end RGB values }
      { Reverse of TopToBottom and LeftToRight directions }
      RGBDifference[0] := GetRValue(BeginColor) - BeginRGBValue[0];
      RGBDifference[1] := GetGValue(BeginColor) - BeginRGBValue[1];
      RGBDifference[2] := GetBValue(BeginColor) - BeginRGBValue[2];
    end;
  end; {case}
  { Calculate the color band's coordinates }
  ColorBand := Rect;
  if Direction in [fdTopToBottom, fdBottomToTop] then begin
{$IFNDEF D2}
    Colors := Math.Max(1, Min(Colors, HeightOf(Rect)));
{$ELSE}
    Colors := Max(1, Min(Colors, HeightOf(Rect)));
{$ENDIF}
    Delta := HeightOf(Rect) div Colors;
  end
  else begin
    Colors := Max(1, Min(Colors, WidthOf(Rect)));
    Delta := WidthOf(Rect) div Colors;
  end;
  with Canvas.Pen do begin { Set the pen style and mode }
    Style := psSolid;
    Mode := pmCopy;
  end;
  { Perform the fill }
  if Delta > 0 then
    for I := 0 to Colors do begin
      case Direction of
        { Calculate the color band's top and bottom coordinates }
        fdTopToBottom, fdBottomToTop: begin
          ColorBand.Top := Rect.Top + I * Delta;
          ColorBand.Bottom := ColorBand.Top + Delta;
        end;
        { Calculate the color band's left and right coordinates }
        fdLeftToRight, fdRightToLeft: begin
          ColorBand.Left := Rect.Left + I * Delta;
          ColorBand.Right := ColorBand.Left + Delta;
        end;
      end; {case}
      { Calculate the color band's color }
      if Colors > 1 then begin
        R := BeginRGBValue[0] + MulDiv(I, RGBDifference[0], Colors - 1);
        G := BeginRGBValue[1] + MulDiv(I, RGBDifference[1], Colors - 1);
        B := BeginRGBValue[2] + MulDiv(I, RGBDifference[2], Colors - 1);
      end
      else begin
        { Set to the Begin Color if set to only one color }
        R := BeginRGBValue[0];
        G := BeginRGBValue[1];
        B := BeginRGBValue[2];
      end;
      with Canvas do begin
        Brush.Color := RGB(R, G, B) or PaletteMask;
        FillRect(ColorBand);
      end;
    end;
  if Direction in [fdTopToBottom, fdBottomToTop] then
    Delta := HeightOf(Rect) mod Colors
  else Delta := WidthOf(Rect) mod Colors;
  if Delta > 0 then begin
    case Direction of
      { Calculate the color band's top and bottom coordinates }
      fdTopToBottom, fdBottomToTop: begin
        ColorBand.Top := Rect.Bottom - Delta;
        ColorBand.Bottom := ColorBand.Top + Delta;
      end;
      { Calculate the color band's left and right coordinates }
      fdLeftToRight, fdRightToLeft: begin
        ColorBand.Left := Rect.Right - Delta;
        ColorBand.Right := ColorBand.Left + Delta;
      end;
    end; {case}
    with Canvas do begin
      case Direction of
        fdTopToBottom, fdLeftToRight:
          Brush.Color := EndColor or PaletteMask;
        fdBottomToTop, fdRightToLeft:
          Brush.Color := BeginColor or PaletteMask;
      end; {case}
      FillRect(ColorBand);
    end;
  end;
end;


function Rechteck(left, top, right, bottom : integer) : TRect;
begin Result.left := left; Result.top :=  top;
      Result.Right :=  right; Result.Bottom :=  bottom;
end;


procedure DrawTiled(DC : HDC; CR: TRect; Bitmap : TBitmap);
var
  Row, Col : Integer;
  NumRows, NumCols: Integer;
  DrawX, DrawY, DrawWidth, DrawHeight: Integer;
  OldRect : TRect;
begin

  NumRows := CR.Bottom DIV Bitmap.Height;
  NumCols := CR.Right DIV Bitmap.Width;
  for Row := 0 to NumRows+1 do
    for Col := 0 to NumCols+1  do
    begin
      DrawX :=  (Col * Bitmap.Width) + CR.Left;
      DrawY :=  (Row * Bitmap.Height) + CR.Top;
      if (DrawX + Bitmap.Width) > CR.Right then
         DrawWidth :=  CR.Right - DrawX
      else
         DrawWidth :=  Bitmap.Width;

      if (DrawY + Bitmap.Height) > CR.Bottom then
         DrawHeight :=  CR.Bottom - DrawY
      else
         DrawHeight := Bitmap.Height;
      GetClipBox(DC, OldRect);
      IntersectCliprect(DC, CR.Left, CR.Top, CR.Right, CR.Bottom);
      BitBlt( DC,
              DrawX, DrawY, DrawWidth, DrawHeight,
              Bitmap.Canvas.Handle,
              0, 0, SRCCOPY);
      IntersectCliprect(DC, OldRect.Left, OldRect.Top, OldRect.Right, OldRect.Bottom);
    end;
end;

procedure DrawFrameBorder(Canvas: TCanvas; X, Y, W, H : Integer);
var
   nx, ny, wi, hi : Integer;
begin

   Canvas.Pen.Width := 1;
   Canvas.Pen.Style := psSolid;
   Canvas.Brush.Style := bsClear;
   Canvas.Pen.Color := clBlack;
   Canvas.Rectangle(0, 0, W, H);

   nx := 1;
   ny := 1;
   wi := W-1;
   hi := H-1;

   Canvas.Pen.Color := clBtnHighlight;
   Canvas.MoveTo(nx, hi-1);
   Canvas.LineTo(nx, ny);
   Canvas.LineTo(wi-1, ny);

   Canvas.Pen.Color := clBtnShadow;
   Canvas.MoveTo(wi-1, ny+1);
   Canvas.LineTo(wi-1, hi-1);
   Canvas.LineTo(nx-1, hi-1);

   Canvas.Pen.Color := clBtnFace;
   Canvas.Pen.Width := 2;
   Canvas.Rectangle(3, 3, wi-1, hi-1);
   Canvas.Pen.Width := 1;

end;

procedure DrawTransparentBitmap( canvas: TCanvas; bmp: TBitmap;
                                 fromRect, toRect : TRect;
                                 cTransparentColor: LongInt);
begin
     Canvas.BrushCopy(fromRect, bmp, toRect, cTransparentColor);
end;


procedure BitmapRotateHorizontal(Bitmap : TBitmap);
VAR aStream : TMemorystream;
    dc:hDC;
    Aptr: Pchar;
    linewidth:INTEGER;

VAR j,k : INTEGER;
 ptr1 : Pchar;
 ptr2 : Pchar;
 aint : LongInt;

BEGIN
   aStream := Tmemorystream.Create;
   aStream.SetSize(  SizeOf (TBITMAPINFOHEADER)
                    +   Bitmap.Height
                     * (Bitmap.Width+4) * 3);
   WITH TBITMAPINFOHEADER(aStream.Memory^) DO
    BEGIN
      biSize := SizeOf(TBITMAPINFOHEADER);
      biWidth := Bitmap.Width;
      biHeight := Bitmap.Height;
      biPlanes := 1;
      biBitCount := 24;
      biCompression := bi_RGB;
      biSizeImage := 0;
      biXPelsPerMeter :=1;
      biYPelsPerMeter :=1;
      biClrUsed :=0;
      biClrImportant :=0;
    END;

    dc := GetDC(0);
    Aptr := Pchar(aStream.Memory) +SizeOf (TBITMAPINFOHEADER);
    GetDIBits(dc,
             Bitmap.Handle,
             0,
             Bitmap.Height,
             Aptr,
             TBITMAPINFO(aStream.Memory^),
             dib_RGB_Colors);

    linewidth := (Bitmap.Width*3);
    linewidth := ((linewidth+3)DIV 4)*4;



    FOR j:=0 TO Bitmap.Height DO
     FOR k:=0 TO Bitmap.Width DIV 2 DO
      BEGIN
       ptr1 := Aptr +
          + j*linewidth
          + k * 3;
        ptr2 := Aptr +
          + j*linewidth
          + (Bitmap.Width-k)*3;
        Move(ptr1^,aint ,3);
        Move(ptr2^,ptr1^,3);
        Move(aint ,ptr2^,3);
      END;



    Bitmap.Handle :=
          CreateDIBitmap(dc,
              TBITMAPINFOHEADER(aStream.Memory^),
              cbm_Init,
              Aptr,
              TBITMAPINFO(aStream.Memory^),
              dib_RGB_Colors);
    aStream.Free;
    ReleaseDC(0,dc);
END;


PROCEDURE BitmapRotateVertical(Bitmap : TBitmap);
VAR aStream : Tmemorystream;
    dc:hDC;
    Aptr: Pchar;
    linewidth:INTEGER;

VAR j : INTEGER;
 ptr1 : Pchar;
 ptr2 : Pchar;
 ptr3 : Pchar;

BEGIN
    aStream := Tmemorystream.Create;
    aStream.SetSize(  SizeOf (TBITMAPINFOHEADER)
                     +   Bitmap.Height
                      * (Bitmap.Width+4) * 3);
    WITH TBITMAPINFOHEADER(aStream.Memory^) DO
     BEGIN
       biSize := SizeOf(TBITMAPINFOHEADER);
       biWidth := Bitmap.Width;
       biHeight := Bitmap.Height;
       biPlanes := 1;
       biBitCount := 24;
       biCompression := bi_RGB;
       biSizeImage := 0;
       biXPelsPerMeter :=1;
       biYPelsPerMeter :=1;
       biClrUsed :=0;
       biClrImportant :=0;
     END;

     dc := GetDC(0);
     Aptr := Pchar(aStream.Memory) +SizeOf (TBITMAPINFOHEADER);
     GetDIBits(dc,
              Bitmap.Handle,
              0,
              Bitmap.Height,
              Aptr,
              TBITMAPINFO(aStream.Memory^),
              dib_RGB_Colors);

     linewidth := (Bitmap.Width*3);
     linewidth := ((linewidth+3)DIV 4)*4;


     GetMem(ptr3,linewidth);
     IF NOT (ptr3 = NIL) THEN
      FOR j:=0 TO Bitmap.Height DIV 2 DO
       BEGIN
        ptr1 := Aptr + j*linewidth;
        ptr2 := Aptr + (Bitmap.Height-j)*linewidth;
        Move(ptr1^,ptr3^,linewidth);
        Move(ptr2^,ptr1^,linewidth);
        Move(ptr3^,ptr2^,linewidth);
       END;

     FreeMem(ptr3);



     Bitmap.Handle :=
           CreateDIBitmap(dc,
               TBITMAPINFOHEADER(aStream.Memory^),
               cbm_Init,
               Aptr,
               TBITMAPINFO(aStream.Memory^),
               dib_RGB_Colors);
     aStream.Free;
     ReleaseDC(0,dc);
END;

PROCEDURE BitmapRotate90(Bitmap : TBitmap);
VAR aStream,bstream : Tmemorystream;
    dc:hDC;
    Aptr,Bptr: Pchar;
    linewidth,blinewidth:INTEGER;

VAR j,k : INTEGER;
 ptr1 : Pchar;
 ptr2 : Pchar;

BEGIN
   aStream := Tmemorystream.Create;
   aStream.SetSize(  SizeOf (TBITMAPINFOHEADER)
                    +   Bitmap.Height
                     * (Bitmap.Width+4) * 3);
   bstream := Tmemorystream.Create;
   bstream.SetSize(  SizeOf (TBITMAPINFOHEADER)
                    +   Bitmap.Width
                     * (Bitmap.Height+4) * 3);
   WITH TBITMAPINFOHEADER(aStream.Memory^) DO
    BEGIN
      biSize := SizeOf(TBITMAPINFOHEADER);
      biWidth := Bitmap.Width;
      biHeight := Bitmap.Height;
      biPlanes := 1;
      biBitCount := 24;
      biCompression := bi_RGB;
      biSizeImage := 0;
      biXPelsPerMeter :=1;
      biYPelsPerMeter :=1;
      biClrUsed :=0;
      biClrImportant :=0;
    END;

   WITH TBITMAPINFOHEADER(bstream.Memory^) DO
    BEGIN
      biSize := SizeOf(TBITMAPINFOHEADER);
      biWidth := Bitmap.Height;
      biHeight := Bitmap.Width;
      biPlanes := 1;
      biBitCount := 24;
      biCompression := bi_RGB;
      biSizeImage := 0;
      biXPelsPerMeter :=1;
      biYPelsPerMeter :=1;
      biClrUsed :=0;
      biClrImportant :=0;
    END;

    dc := GetDC(0);
    Aptr := Pchar(aStream.Memory) +SizeOf (TBITMAPINFOHEADER);
    Bptr := Pchar(bstream.Memory) +SizeOf (TBITMAPINFOHEADER);
    GetDIBits(dc,
             Bitmap.Handle,
             0,
             Bitmap.Height,
             Aptr,
             TBITMAPINFO(aStream.Memory^),
             dib_RGB_Colors);

    linewidth := (Bitmap.Width*3);
    linewidth := ((linewidth+3)DIV 4)*4;
    blinewidth := (Bitmap.Height*3);
    blinewidth := ((blinewidth+3)DIV 4)*4;


    FOR j:=0 TO Bitmap.Height DO
    FOR k:=0 TO Bitmap.Width DO
     BEGIN
      ptr1 := Aptr + j*linewidth + k * 3;
      ptr2 := Bptr + k* blinewidth + (Bitmap.Height-j)*3;
      Move(ptr1^,ptr2^,3);
     END;


    Bitmap.Handle :=
          CreateDIBitmap(dc,
              TBITMAPINFOHEADER(bstream.Memory^),
              cbm_Init,
              Bptr,
              TBITMAPINFO(bstream.Memory^),
              dib_RGB_Colors);
    aStream.Free;
    bStream.Free;
    ReleaseDC(0,dc);
END;


type PByte  = ^Byte;

procedure BitmapGray(Bitmap : TBitmap);
VAR aStream : Tmemorystream;
    dc:hDC;
    Aptr: PChar;
    linewidth:INTEGER;

VAR j, k, e : INTEGER;
 ptr : PChar;
 tc, ac : TMyColor;
 y: Integer;


 procedure getColor(p : PChar; var c : TMyColor);
 begin
   c.B := Byte(p^);
   c.G := Byte((p+1)^);
   c.R := Byte((p+2)^);
 end;

 procedure setColor(p : PChar; var c : TMyColor);
 begin
   p^ := Char(c.B);
   (p+1)^ := Char(c.G);
   (p+2)^ := Char(c.R);
 end;


BEGIN
   aStream := Tmemorystream.Create;
   aStream.SetSize(  SizeOf (TBITMAPINFOHEADER)
                    +   Bitmap.Height
                     * (Bitmap.Width+4) * 3);
   WITH TBITMAPINFOHEADER(aStream.Memory^) DO
    BEGIN
      biSize := SizeOf(TBITMAPINFOHEADER);
      biWidth := Bitmap.Width;
      biHeight := Bitmap.Height;
      biPlanes := 1;
      biBitCount := 24;
      biCompression := bi_RGB;
      biSizeImage := 0;
      biXPelsPerMeter :=1;
      biYPelsPerMeter :=1;
      biClrUsed :=0;
      biClrImportant :=0;
    END;

    dc := GetDC(0);
    Aptr := PChar(aStream.Memory) +SizeOf (TBITMAPINFOHEADER);
    GetDIBits(dc,
             Bitmap.Handle,
             0,
             Bitmap.Height,
             Aptr,
             TBITMAPINFO(aStream.Memory^),
             dib_RGB_Colors);

    linewidth := (Bitmap.Width*3);
    linewidth := ((linewidth+3)DIV 4)*4;
    tc := TMyColor(Bitmap.TransparentColor);

    For j:= 0 TO Bitmap.Height do begin
      k := (j*lineWidth);
      e := (j+1)* lineWidth;
      while k <  e do begin
          ptr := APtr + k;
          Inc(k, 3);
          getColor(ptr, ac);
          if (tc.R <> ac.R) OR (tc.G <> ac.G) OR (tc.B <> ac.B) then begin
           y := (ac.R + ac.G + ac.B) div 3;
           if y > 200 then y := 200;
           ac.R := y;
           ac.G := y;
           ac.B := y;
           setColor(ptr, ac);
           {
            else Inc(ac.R,180);
           if (ac.G + 180) > 255 then ac.G := 255
            else Inc(ac.G,180);
            setColor(ptr, ac);
            }
           { if (val + 100) > 255 then val := 255
            else Inc(val,100);}
           {if (ac.R + 100) > 255 then ac.R := 255
            else Inc(ac.R,100);
           if (ac.G + 100) > 255 then ac.G := 255
            else Inc(ac.G,100);
           if (ac.B + 100) > 255 then ac.B := 255
            else Inc(ac.B,100);
            setColor(ptr, ac);
            }

{
            y :=  (ac.R+ac.G+ac.B)/ 1.02;
            u := (y-ac.R) *1.8;
            v := (y-ac.G) *1.8;
            x := (y-ac.B) *1.8;
            ac.R := Round(y-u);
            ac.G := Round(y-v);
            ac.B := Round(y-x);
            SetColor(ptr, ac);
 }
            {if (k > 0) then begin
                ptr := APtr + (k-3);
                getColor(ptr, ac);
                if (tc.R = ac.R) AND (tc.G = ac.G) AND (tc.B = ac.B) then begin
                  ac.R := 255; ac.G := 255; ac.B := 0;
                  setColor(ptr, ac)
                end;
            end;
             }
          end;
      end;
    end;

        {
    FOR j:=0 TO Bitmap.Height * linewidth DO begin
        ptr1 := Aptr + j;
        val := Byte(ptr1^);
        if (val + 100) > 255 then val := 255
        else Inc(val,100);
        ptr1^ := Chr(val);
     END;
     }

    Bitmap.Handle :=
          CreateDIBitmap(dc,
              TBITMAPINFOHEADER(astream.Memory^),
              cbm_Init,
              Aptr,
              TBITMAPINFO(astream.Memory^),
              dib_RGB_Colors);
    aStream.Free;
    ReleaseDC(0,dc);
END;

end.
