{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }                                                                              
{                                                                              }
{ Filename           : DFWSoundPropertyEditor.pas                              }
{ Short description  : Sound property editor & Sound Category (D5)             }
{ Author             : Carsten Butzek                                          }
{ E-Mail             : cabutzek@muenster.de                                    }
{ Internet           : http://www.delphifreeware.de                            }
{ Version            : 1.0                                                     }
{ Date last modified : 04/04/2000                                              }
{ Delphi versions    : 2,3,4,5                                                 }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       :                                                           }
{ Classes          : TDFWSoundCategory, TDFWInternetCategory,                  }
{                    TDFWSoundProperty, TDFWSoundSelectForm                    }
{ Types            :                                                           }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWSoundPropertyEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, Registry, DFWSound, DsgnIntf;

type
{$IFDEF D5}
  TDFWSoundCategory = class (TPropertyCategory)
    class function Name: string; override;
    class function Description: string; override;
  end;

  TDFWInternetCategory = class (TPropertyCategory)
    class function Name: string; override;
    class function Description: string; override;
  end;

{$ENDIF}
  TDFWSoundProperty = class (TPropertyEditor)
  public
    function GetAttributes:TPropertyAttributes; override;
    procedure Edit; override;
  end;

  TDFWSoundSelectForm = class(TForm)
    Button1: TButton;
    Button2: TButton;
    GroupBox1: TGroupBox;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    RadioButton3: TRadioButton;
    CheckBox1: TCheckBox;
    WaveEdit: TEdit;
    OpenFileButton: TSpeedButton;
    RadioButton4: TRadioButton;
    Label1: TLabel;
    ResourceEdit: TEdit;
    WindowsComboBox: TComboBox;
    PlaySoundButton: TBitBtn;
    OpenDialog: TOpenDialog;
    procedure RadioButton1Click(Sender: TObject);
    procedure WaveEditEnter(Sender: TObject);
    procedure ResourceEditEnter(Sender: TObject);
    procedure WindowsComboBoxEnter(Sender: TObject);
    procedure OpenFileButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PlaySoundButtonClick(Sender: TObject);
    procedure WaveEditChange(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    SoundInfo:TDFWSoundInfo;
  end;

implementation
uses MMSystem, FileCTRL;
{$R *.DFM}

{$IFDEF D5}
class function TDFWSoundCategory.Name: string;
begin
     Result:='Sound';
end;

class function TDFWSoundCategory.Description: string;
begin
     Result:='DFW Sound properties';
end;

class function TDFWInternetCategory.Name: string;
begin
     Result:='Internet';
end;

class function TDFWInternetCategory.Description: string;
begin
     Result:='DFW Internet properties';
end;

{$ENDIF}

procedure TDFWSoundSelectForm.RadioButton1Click(Sender: TObject);
begin
     WaveEdit.Color:=clBtnFace;
     ResourceEdit.Color:=clBtnFace;
     WindowsComboBox.Color:=clBtnFace;
     PlaySoundButton.Enabled:=false;
     if RadioButton1.checked then begin
        WaveEdit.Color:=clWindow;
        PlaySoundButton.enabled:=FileExists(WaveEdit.Text);
     end;
     if RadioButton2.checked then begin
        ResourceEdit.Color:=clWindow;
     end;
     if RadioButton3.checked then begin
        WindowsComboBox.Color:=clWindow;
        PlaySoundButton.enabled:=true;
     end;
end;

procedure TDFWSoundSelectForm.WaveEditEnter(Sender: TObject);
begin
     RadioButton1.checked:=true;
end;

procedure TDFWSoundSelectForm.ResourceEditEnter(Sender: TObject);
begin
     RadioButton2.checked:=true;
end;

procedure TDFWSoundSelectForm.WindowsComboBoxEnter(Sender: TObject);
begin
     RadioButton3.checked:=true;
end;

procedure TDFWSoundSelectForm.OpenFileButtonClick(Sender: TObject);
begin
     if OpenDialog.Execute then begin
        RadioButton1.checked:=true;
        if WaveEdit.Text<>'' then
           if DirectoryExists(ExtractFilePath(WaveEdit.Text)) then begin
              OpenDialog.InitialDir:=ExtractFilePath(WaveEdit.Text);
              OpenDialog.FileName:=ExtractFileName(WaveEdit.Text);
           end;
        WaveEdit.Text:=OpenDialog.FileName;
     end;
end;

procedure TDFWSoundSelectForm.FormCreate(Sender: TObject);
var Reg:TRegistry;
begin
     WindowsComboBox.Items.Clear;
     Reg:=TRegistry.create;
     try
       Reg.RootKey := HKEY_USERS;
       if Reg.OpenKey('\.DEFAULT\AppEvents\EventLabels', false)
          then Reg.GetKeyNames(WindowsComboBox.Items);
     finally
       Reg.CloseKey;
       Reg.Free;
       inherited;
     end;

     WindowsComboBox.ItemIndex:=0;
end;

procedure TDFWSoundSelectForm.PlaySoundButtonClick(Sender: TObject);
begin
     PlaySound(nil, HInstance, SND_PURGE);
     if RadioButton1.checked then
        if FileExists (WaveEdit.Text) then
           PlaySound(PChar(WaveEdit.Text), HInstance,SND_FILENAME or SND_ASYNC)
        else MessageDLG('File does not exist!', mtError, [mbok], 0);
     if RadioButton3.checked then begin
        PlaySound(PChar(WindowsComboBox.Text), HInstance, SND_ASYNC);
     end;
end;

procedure TDFWSoundSelectForm.WaveEditChange(Sender: TObject);
begin
     PlaySoundButton.enabled:=FileExists(WaveEdit.Text);
end;

function TDFWSoundProperty.GetAttributes:TPropertyAttributes;
begin
     Result:=[paMultiSelect, paDialog, paReadOnly];
end;

procedure TDFWSoundProperty.Edit;
var
  DFWSoundSelectForm: TDFWSoundSelectForm;
  i:integer;
begin
     DFWSoundSelectForm:=TDFWSoundSelectForm.create(application);
     with DFWSoundSelectForm do begin
       SoundInfo:=TDFWSoundInfo(GetOrdValue);
       case SoundInfo.SelectedIndex of
         0:RadioButton4.Checked:=true;
         1:RadioButton1.Checked:=true;
         2:RadioButton2.Checked:=true;
         3:RadioButton3.Checked:=true;
       end;
       WaveEdit.Text:=SoundInfo.FileName;
       CheckBox1.checked:=SoundInfo.RelativePath;
       ResourceEdit.Text:=SoundInfo.ResourceID;
       if SoundInfo.WindowsSound<>'' then
          for i:=0 to WindowsComboBox.items.count-1 do
            if Uppercase(SoundInfo.WindowsSound)=Uppercase(WindowsComboBox.items[i]) then begin
               WindowsComboBox.ItemIndex:=I;
               break;
            end;

       if ShowModal=mrOK then begin
          if RadioButton1.Checked then SoundInfo.SelectedIndex:=1;
          if RadioButton2.Checked then SoundInfo.SelectedIndex:=2;
          if RadioButton3.Checked then SoundInfo.SelectedIndex:=3;
          if RadioButton4.Checked then SoundInfo.SelectedIndex:=0;
          SoundInfo.FileName:=WaveEdit.Text;
          SoundInfo.RelativePath:=CheckBox1.checked;
          SoundInfo.ResourceID:=ResourceEdit.Text;
          SoundInfo.WindowsSound:=WindowsComboBox.Text;
          SetOrdValue(Longint(SoundInfo));
       end;
       Free;
     end;
end;

end.
